#!/usr/bin/env python
if __name__ == '__main__':

    #  _             
    # |_) _. _|_ |_  
    # |  (_|  |_ | | 
    #
    import sys
    path_config = sys.argv[1]

    #                                     _       
    # |_|  _. ._   _| |  _     _  _  ._ _|_ o  _  
    # | | (_| | | (_| | (/_   (_ (_) | | |  | (_| 
    #                                          _|
    from zezfio import generator
    template = generator.generate_fortran(path_config)

    print template


#  o  _  _  ._  
#  | _> (_) | | 
# _|           
#
import json
def _byteify(data, ignore_dicts=False):
    # if this is a unicode string, return its string representation
    if isinstance(data, unicode):
        return data.encode('utf-8')
    # if this is a list of values, return list of byteified values
    if isinstance(data, list):
        return [_byteify(item, ignore_dicts=True) for item in data]
    # if this is a dictionary, return dictionary of byteified keys and values
    # but only if we haven't already byteified it
    if isinstance(data, dict) and not ignore_dicts:
        return {
            _byteify(key,
                     ignore_dicts=True): _byteify(value,
                                                  ignore_dicts=True)
            for key, value in data.iteritems()
        }
    # if it's anything else, return it in its original form
    return data

def json_load_byteified(file_handle):
    return _byteify(
        json.load(file_handle,
                  object_hook=_byteify),
        ignore_dicts=True)

def load_json(path):
    with open(path) as data_file:
        data = json_load_byteified(data_file)
    return data

#                                 
# _|_  _  ._ _  ._  |  _. _|_  _  
#  |_ (/_ | | | |_) | (_|  |_ (/_ 
#               |                 

import os
file_dir = os.path.dirname(__file__)
template_dir_rel = os.path.join(file_dir,"templates")
template_dir = os.path.abspath(template_dir_rel)

from jinja2 import Environment, FileSystemLoader
env = Environment(loader=FileSystemLoader(template_dir))


def generate_server(json_path,db_path,debug=False):

    json_config = load_json(json_path)
    template = env.get_template('management.jinja2.py').render(json_config=json_config,
                                                               db_path=db_path)
    if debug:
        print template

    return template

def generate_fortran(json_path,debug=False):

    from zezfio.babel import c2stuff
    json_config = load_json(json_path)
    template = env.get_template('lib_fortran.jinja2.f90').render(json_config=json_config,
                                                                 c2stuff=c2stuff)
    if debug:
        print template

    return template

def importCode(code,name):

    import sys,imp

    module = imp.new_module(name)
    compiled_code = compile(code, '<string>', 'exec')
    exec compiled_code in module.__dict__
    sys.modules["module"] = module

    return module

def get_dict_module_server(json_path,db_path,debug=False):

    code = generate_server(json_path,db_path,debug)
    m = importCode(code, "zezfio_server")
    return m.d_instance



#!/usr/bin/env python

from zezfio.babel import type_fortran2c 
from collections import defaultdict

import re
import sys

magic_split_line_regex_dim=re.compile(r' +(\w*) +(.*?)( \(.*\) ?)?$')
magic_split_line_regex_formula=re.compile(r'^ +(\w+) +(.*?)\s*=(.+)?$')


def one_dwarf(path):
    with open(path,"r") as f:
        data = f.read()
    
    l_block = [block for block in re.split('\n\s*\n', data) if block]
    
    json_data = defaultdict(dict)
    
    for block in l_block:
        l_ligne = [ i for i in block.split("\n") if i ]

        category = l_ligne[0]
    
        l_attribute = []
    
        for attribute in l_ligne[1:]:
    
            info = dict()
            
            if "=" not in attribute:
                l_var = [v.strip() for v in re.findall(magic_split_line_regex_dim,attribute)[0] if v]

                info["name"] = l_var[0]
                info["type"] = type_fortran2c(l_var[1])
        
                try:
                    info["shape"] = "[%s]" % l_var[2][1:-1]
                except IndexError:
                    pass
            else:

                l_var = re.findall(magic_split_line_regex_formula,attribute)[0]

                info["name"] = l_var[0]
                info["type"] = type_fortran2c(l_var[1])
                info["default"] = l_var[2]
                info["default"] = info["default"].replace("maxval","max")
                info["default"] = info["default"].replace("minval","min")
        
            l_attribute.append(info)
        
        json_data[category]["attributes"] = l_attribute

    return json_data


def stz_dim_array(shape, list_category):
    for category in list_category:
        old = "%s_"%category
        new = "%s."%category
        shape = re.sub(r"\b%s"%old,new,shape)

    return shape



def schneewittchen(l_file):
    d = dict()
    for path in l_file:   
        d.update(one_dwarf(path))  

    return d

def magic_mirror(data):
    list_category = data.keys()
    for category,attributes in data.iteritems():
        for var in attributes["attributes"]:

            try:
                var["shape"] = stz_dim_array(var["shape"],list_category)
            except:
                pass

            try:
                var["default"] = stz_dim_array(var["default"],list_category)
            except KeyError:
                pass

if __name__ == '__main__':
    

    data = schneewittchen(sys.argv[1:])
    magic_mirror(data)

    import json
    print json.dumps(data,  indent = 4)



#!/usr/bin/env python
import zmq
import logging

import socket

def get_ip_address():
 return socket.gethostname()


if __name__ == '__main__':
    #                         
    # |  _   _   _  o ._   _  
    # | (_) (_| (_| | | | (_| 
    #        _|  _|        _| 
    #        
    logging.basicConfig(filename='zezfio.log', level=logging.ERROR)

    #  _             
    # |_) _. _|_ |_  
    # |  (_|  |_ | | 
    #
    import sys, os
    address, path_config, db_path_rel = sys.argv[1:]
    db_path = os.path.abspath(db_path_rel)

    #                                     _       
    # |_|  _. ._   _| |  _     _  _  ._ _|_ o  _  
    # | | (_| | | (_| | (/_   (_ (_) | | |  | (_| 
    #                                          _|
    from zezfio import generator
    d_instance = generator.get_dict_module_server(path_config,db_path)

    #  __                 
    # (_   _  ._    _  ._ 
    # __) (/_ | \/ (/_ |  
    #
    context = zmq.Context(io_threads=4)
    sock = context.socket(zmq.REP)

    #IPC Adress
    sock.bind(address)

    #Random tcp
    tcp_port_selected = sock.bind_to_random_port('tcp://*', min_port=6001, max_port=6004, max_tries=100)
    # ~#~#~#~#
    # Little_optimisation
    # ~#~#~#~#
    send = sock.send
    recv = sock.recv

    # ~#~#~#~#
    # Error_code
    # ~#~#~#~#    
    from ctypes import c_int
    errno_fail = c_int(-1)
    errno_success = c_int(0)

    while True:
            #For the sake of performance, inline the main loop
            #Get the info
            try:
                m = recv()
            except zmq.error.ZMQError:
                logging.exception("Error when asking for a message")
                continue

            if m == "get_tcp":
                send("tcp://%s:%s" %(get_ip_address(),tcp_port_selected))
                continue

            try:
                action, str_instance, name = m.split(".")
            except:
                logging.exception("Error when splitting the message %s"%m)
                continue

            try:
                instance = d_instance[str_instance]
            except Exception as e:
                logging.exception("Cannot get the %s category" % str_instance)
                send(errno_fail)
                continue

            #Get the instance
            if action == "has":

                try:
                    getattr(instance,"%s_interface" % name)
                except Exception as e:
                    logging.exception(e)
                    send(errno_fail)
                    continue
                else:
                    send(errno_success)
                    logging.debug("Is Present")

            elif action == "size":

                try:
                    size = getattr(instance, "%s_len_interface" % name)
                except Exception as e:
                    logging.error(e)
                    send(errno_fail)
                    continue
                else:
                    send(errno_success,  zmq.SNDMORE)
                    send(size)
                    logging.debug("Size send")

            elif action == "get":
                try:
                    size = getattr(instance, "%s_len_interface" % name)
                    array = getattr(instance, "%s_interface" % name)
                except Exception as e:
                    logging.exception(e)
                    send(errno_fail)
                    continue
                else:
                    send(errno_success,  zmq.SNDMORE)
                    send(size, zmq.SNDMORE)               
                    send(array)

            elif action == "set":
                try:
                    bytes = recv()
                except zmq.error.ZMQError:
                    logging.exception("Error when reading the number of bytes in set")
                    continue

                try:
                    getattr(instance, "set_%s" % name)(bytes)
                except Exception as e:
                    logging.exception(e)
                    send(errno_fail)
                    continue
                else:
                    errno_success = d_instance["zezfio_id"]
                    send(errno_success)
                    logging.debug("Set Done. Errno is %s" % errno_success.value)

            elif action == "get_ascii":
                try:
                    array = getattr(instance, "%s_ascii" % name)
                except Exception as e:
                    logging.exception(e)
                    send(errno_fail)
                    continue
                else:
                    send(errno_success,  zmq.SNDMORE)            
                    send(array)

            elif action == "set_ascii":
                try:
                    string = recv()
                except zmq.error.ZMQError:
                    logging.exception("Error when reading the number of string in set")
                    continue

                try:
                    getattr(instance, "set_ascii_%s" % name)(string)
                except Exception as e:
                    logging.exception(e)
                    send(errno_fail)
                    continue
                else:
                    errno_success = d_instance["zezfio_id"]
                    send(errno_success)
                    logging.debug("Set Done. Errno is %s" % errno_success.value)

            else:
                logging.error("Cannot understand the action to do %s"%action)
                continue



from setuptools import setup, Extension

module = Extension('gz2ar',
                    libraries = ['z'],
                    sources = ['./zezfio/io/gz2ar.c'])

setup(
  name = 'zezfio',
  packages = ['zezfio'],
  scripts = ["zlegacy2json", "zfang", "zserver"],
  version = '0.9',
  description = 'The lovely client/server version of Ezfio',
  author = 'Thomas applencourt',
  author_email = 'applencourt@irsamc.ups-tlse.fr',
  url = 'https://github.com/TApplencourt/Zezfio2',
  download_url = 'https://github.com/TApplencourt/Zezfio2/archive/master.zip', 
  keywords = ['programming', 'fortran', 'EZFIO','ZMQ'],
  install_requires=['irpy','pyzmq', 'jinja2'],
  ext_modules = [module])
